#ifndef MATRIX_H
#define MATRIX_H

#include <assert.h>

template < class T >
class Matrix
{
public:
	Matrix( int = 5, int = 5);                    //Default constructor
	Matrix( const Matrix &);					  //Copy constructor
	~Matrix () { delete [] ptr; }
	int getR() const { return Row; }
	int getC() const { return Column;}
	class MatrixRow								  //A proxy class used to overload [][]
	{
	public:
		T& operator [] ( int j)
		{
			assert( 0 <= j && j < Matrix().getC() );      // Check for subscript out of range
			return *( Rownum + j);
		}
		T* Rownum;
	};

	MatrixRow operator [] ( int i)                // double subscript operator
	{
		MatrixRow temp;
		assert( 0 <= i && i < Matrix::getR() );   // Check for subscript out of range
		temp.Rownum = ptr + i * Column; 
		return temp;
	}
	bool operator==( const Matrix & ) const;            // Comparing classes
	// Detemine if two Matrixes  are not equal
	bool operator!=( const Matrix & ) const
	{ return !( *this == right ); }
	const Matrix& operator = ( const Matrix &);

private:
	int Row;
	int Column;
	T* ptr;            //pointer to the first element
};

//Default constructor
template < class T>
Matrix<T> :: Matrix( int i, int j)
{
	ptr = new T [ i * j ];               //Create space for array
	assert ( ptr != 0);
	Row = ( i > 0 ? i :5 );
	Column = ( j > 0 ? j : 5);
//	for( int x = 0;x < Row * Column ;x++)
//		ptr[ x ] = 0;               //Initialize array
}

template< class T >
Matrix<T> :: Matrix( const Matrix<T> & init) : Row( init.Row),Column( init.Column)
{
	ptr = new T [Row * Column];      //Create space for array
	assert ( ptr != 0 );
	for ( int x = 0; x < Row*Column; x++)
		ptr[x] = init.ptr[x];
}

template< class T >
bool Matrix<T>::operator == ( const Matrix<T>& right ) const
{
	if ( ( Row != right.Row) || ( Column != right.Column) ) return false;
	 for ( int x = 0; x < Row*Column; x++)
		 if ( ptr[ x ] != right.ptr[ x ] ) return false;
	return true;
}

// Overloaded assignment operator
template< class T >
const Matrix<T>& Matrix<T>::operator = ( const Matrix<T> & right)
{
	if ( &right != this )        // Checks for self-assignment
	{
		delete [] ptr;				   // Reclaim space for last Matrix
		Row = right.Row;
		Column = right.Column;
		ptr = new T [ Row * Column ];    // Create space for new array
		assert( ptr != 0 );
		for ( int x = 0; x < Row * Column; x++)
			ptr[ x ]=right.ptr[ x ] ;
	}
	return *this;
}


#endif MATRIX_H